(
var createTransportPattern;

~bergerBusArray = 6.collect({Bus.audio(s, 1)});

Event.addEventType(\osc, {
	if (~addr.postln.notNil) {
		~msg.postln;
		~addr.sendMsg(~path, *~msg);
	};
});


//create synth defs
~bergerCreateSynthsForLive = {arg b;
	// this creates a different waveform for each sonification of each part
	Buffer.freeAll(s);
	b = 8.collect({var buf = Buffer.alloc(s, 512, 1); buf.sine1(1.0 / 5.collect({arg i; pow(i + 1, 5.0.rand + 1)}), true, true, true)});

	SynthDef(\berger, {arg freq, amp = 0.2, del = 5, gate = 1, sustain = 1, buf = 0, out = 0;
		Out.ar(out, Osc.ar(Select.kr(buf, b), freq, 0, amp) * EnvGen.kr(Env.adsr(0.05, sustain, 0.25, 0.1), gate, 1, 0, 1, doneAction: 2));
	}).add;
};

~robinsonCreateSynthsForLive = {arg b;
	Buffer.freeAll(s);
	b = 8.collect({var buf = Buffer.alloc(s, 512, 1); buf.sine1(1.0 / 5.collect({arg i; pow(i + 1, 5.0.rand + 1)}), true, true, true)});

	SynthDef(\robinson, {arg freq, amp = 0.2, del = 5, gate = 1, sustain = 1, buf = 0, out = 0;
		Out.ar(out, Osc.ar(Select.kr(buf, b), freq, 0, amp) * EnvGen.kr(Env.sine(sustain + 0.01), gate, 1, 0, 1, doneAction: 2));
	}).add;
};

~penroseCreateSynthsForLive = {arg b;
	Buffer.freeAll(s);
	b = 8.collect({var buf = Buffer.alloc(s, 512, 1); buf.sine1(1.0 / 5.collect({arg i; pow(i + 1, 5.0.rand + 1)}), true, true, true)});

	SynthDef(\penrose_ins, {arg freq, amp = 0.2, del = 5, gate = 1, sustain = 1, buf = 0, out = 0;
		Out.ar(out /*10 + buf*/, Osc.ar(Select.kr(buf, b), freq, 0, amp) * EnvGen.kr(Env.adsr(1, sustain - 0.3, 0.7, 0.5), gate, 1, 0, 1, doneAction: 2));
	}).add;

	SynthDef(\penrose_fades, {arg subSeqLength, gate = 1, ins = 0, out = 0;
		Out.ar(out, In.ar(10 + ins) * EnvGen.kr(Env.new([0.4, 0.8, 0.8, 0.4, 0], [3, subSeqLength - 6, 3, 0.1], \sine), gate, 1, 0, 1, doneAction: 2));
	}).add;
};


//create patterns
~bergerCreatePatternsForLive = {arg seqs, group;
	var pBinds;
	pBinds = seqs.collect({arg r, i;
		Pbind(\instrument, \berger,
			//\buf, Pseq((i - ((0..(r.slice(nil, 2).flat.size - 1)) / 10).trunc.asInteger) % 8),
			\group, group,
			\buf, i,
			\out, ~bergerBusArray[i],
			//\freq, Pseq(55 * r[1]),
			\freq, Pseq((55 * r[1]).cpsmidi.round(0.5).midicps),
			\dur, Pseq(r[0]),
			\sustain, Pseq(r[0]),
			\amp, Pseq(r[2] / 1) //4)
		)
	});
};

~robinsonCreatePatternsForLive = {arg seqs, group;
	var pBinds;
	seqs = File.readAllString((~dir +/+ "music_data" +/+ "robinson.txt").standardizePath).interpret;
	pBinds = seqs[0].collect({arg r, i;
		Pbind(\instrument, \robinson,
			\buf, i,
			\freq, Pseq(12.midicps * r.slice(nil, 1)),
			\dur, Pseq(r.slice(nil, 0) * 0.25 * (3/4)),
			\sustain, Pseq(r.slice(nil, 0) * 0.25 * (3/4)),
			\amp, Pseq(r.slice(nil, 2))
		)
	});
};

~penroseCreatePatternsForLive = {arg seqs, group;
	var finSeqs, finFades;
	finSeqs = seqs[0].collect({arg r, i;
		Pbind(\instrument, \penrose_ins,
			\group, group,
			\buf, i,
			//\freq, Pseq((24.midicps * r[1]).cpsmidi.round(0.5).midicps),
			\freq, Pseq((24.midicps * r[1])),
			\dur, Pseq(r[0] * 0.125),
			\sustain, Pseq(r[0] * 0.125),
			\amp, Pseq(((r[2] * 0.25) / pow(1.5, ((i % 6) + 1))))
		)
	});

	finFades = seqs[2].collect({ arg r, i;
		Pbind(\instrument, \penrose_fades,
			\group, group,
			\dur, Pseq(r * 0.125),
			\subSeqLength, Pseq(r * 0.125),
			\ins, i
		)
	});

	finSeqs /*++ finFades*/;
};




SynthDef(\mixer, {arg freq, gate = 1, sustain, amp, dur, out;
	var nameSpaces, sigs;

	sigs = [~bergerBusArray].collect({arg busArray, i;
		var nameSpace, sig;
		nameSpace = ['berger', 'sine', 'bass', 'hdust', 'sampler'][i];
		sig = busArray.collect({arg bus, c; In.ar(bus, 1) * NamedControl.kr(\ ++ nameSpace ++ '_volume_' ++ c, 1, 0.1)});
		sig = sig.collect({arg channel, c; Pan2.ar(channel, NamedControl.kr(\ ++ nameSpace ++ '_pan_' ++ c, i / (busArray.size - 1), 0.1) * 2 - 1)});
		sig = sig.collect({arg channel, c; channel * NamedControl.kr(\ ++ nameSpace ++ '_mute_' ++ c, 1, 0.1)});
		sig = Mix.ar(sig) * pow(NamedControl.kr(\ ++ nameSpace ++ '_volume_master', 1, 0.5), 2);
	});

	sigs = Mix.ar(sigs / 6);
	Out.ar(0, sigs)
}).add;

)

(

var createTransportData, createTransportPattern, group, berger, mixer, measureLengths, seqs, transportData, playbackData, player;

createTransportData = {arg measureLengths;
	measureLengths.collect({arg beats, measure; beats.asInteger.collect({arg beat; [measure + 1, beat + 1]})}).flatten;
};

createTransportPattern = {arg addr, transportData;
	Pbind(
		\type, \osc,
		\addr, addr,
		\path, "/playing",
		\msg, Pseq(transportData, 1),
		\dur, 1
	);
};

~bergerCreateSynthsForLive.value;
~robinsonCreateSynthsForLive.value;
~penroseCreateSynthsForLive.value;

group = Group.new;
berger = Synth.tail(group, \berger);
mixer = Synth.tail(group, \mixer);

playbackData = Dictionary.new(n: 6);

seqs = File.readAllString((~dir +/+ "music_data" +/+ "berger.txt").standardizePath).interpret;
measureLengths = seqs[0][6];
playbackData.add(\berger -> [~bergerCreatePatternsForLive.value(seqs, group), createTransportData.value(measureLengths), measureLengths]);

seqs = File.readAllString((~dir +/+ "music_data" +/+ "robinson.txt").standardizePath).interpret;
measureLengths = seqs[1].sum.collect({4});
playbackData.add(\robinson -> [~robinsonCreatePatternsForLive.value(seqs, group), createTransportData.value(measureLengths), measureLengths]);

seqs = File.readAllString((~dir +/+ "music_data" +/+ "penrose.txt").standardizePath).interpret;
measureLengths = seqs[1].sum.collect({4});
playbackData.add(\penrose -> [~penroseCreatePatternsForLive.value(seqs, group), createTransportData.value(measureLengths), measureLengths]);


OSCdef(\mixer, {arg msg, time, addr, port;
	msg.postln;
	mixer.set((msg[1] ++ '_' ++ msg[2] ++ '_' ++ msg[3]), msg[4])
}, \mixer);

OSCdef(\transport, {arg msg, time, addr, port;
	msg.postln;
	if(msg[1].postln == 0, {
		group.set(\release, 2);
		group.set(\gate, 0);
		player.stop;
	}, {
		var pbinds, transportData, measureLengths, patterns, stream, offset, offsetStream, terminationStream;
		# pbinds, transportData, measureLengths = playbackData[msg[2]];
		patterns = Ppar(pbinds.add(createTransportPattern.value(addr, transportData)));

		stream = patterns.asStream;
		offset = stream.fastForward(measureLengths.keep(msg[3].asInteger - 1).sum);

		offsetStream = Routine({offset.wait});
		terminationStream = Routine({addr.sendMsg("/transport", 0)});

		player = EventStreamPlayer(offsetStream ++ stream ++ terminationStream);
		player.play;
	});
}, \transport);

)

~penroseMusic[1]

/*
~bergerCreateSynthsForLive.value;
~patterns = ~bergerCreatePatternsForLive.value(~bergerMusic);
~patterns.play
~bergerMusic[0][6].collect({arg beats, measure; beats.asInteger.collect({arg beat; [measure + 1, beat + 1]})}).flatten;
*/
